// Copyright (c) 2019, the Dart project authors.  Please see the AUTHORS file
// for details. All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

/// @assertion We say that a type T0 is a subtype of a type T1 (written T0 <: T1)
/// when:
/// Right Object: if T1 is Object then:
///  - if T0 is an unpromoted type variable with bound B then T0 <: T1 iff
///      B <: Object
///  - if T0 is a promoted type variable X & S then T0 <: T1 iff S <: Object
///  - if T0 is FutureOr<S> for some S, then T0 <: T1 iff S <: Object.
///  - if T0 is S* for any S, then T0 <: T1 iff S <: T1
///  - if T0 is Null, dynamic, void, or S? for any S, then the subtyping does not
///      hold (per above, the result of the subtyping query is false).
///  - Otherwise T0 <: T1 is true.
/// @description Check that if T0 is a promoted type variable X & S and
/// S <: Object then T0 is subtype of Object even is X is not subtype of Object
/// @author sgrekhov@unipro.ru
///
/// @description Check that if type T0 is a subtype of a type T1, then instance
/// of T0 can be used as an argument of type T1. Test superclass members
/// @author sgrekhov@unipro.ru
///
/// This test is generated from test_types/right_object_A03.dart and
/// test_cases/arguments_binding_x02.dart. Don't modify it!
/// If you need to change this test, then change one of the files above and then
/// run generator/generator.dart to regenerate the tests.

// Requirements=nnbd-weak
import "../../utils/legacy_lib.dart";
class S extends X {}

S t0Instance = new S();
Object t1Instance = new Object();

const t1Default = const Object();

class ArgumentsBindingSuper1_t02 {
  Object m = t1Default;

  ArgumentsBindingSuper1_t02(Object value): m = value {}
  ArgumentsBindingSuper1_t02.named(Object value, {Object val2 = t1Default}): m = value {}
  ArgumentsBindingSuper1_t02.positional(Object value, [Object val2 = t1Default]): m = value {}
  ArgumentsBindingSuper1_t02.short(this.m);

  void superTest(Object val) {}
  void superTestPositioned(Object val, [Object val2 = t1Default]) {}
  void superTestNamed(Object val, {Object val2 = t1Default}) {}
  Object get superGetter => m;
  void set superSetter(Object val) {}
}

class ArgumentsBinding1_t02 extends ArgumentsBindingSuper1_t02 {
  ArgumentsBinding1_t02(dynamic t1) : super(t1) {}
  ArgumentsBinding1_t02.c1(dynamic t1) : super.named(t1) {}
  ArgumentsBinding1_t02.c2(dynamic t1, dynamic t2) : super.named(t1, val2: t2) {}
  ArgumentsBinding1_t02.c3(dynamic t1) : super.positional(t1) {}
  ArgumentsBinding1_t02.c4(dynamic t1, dynamic t2) : super.positional(t1, t2) {}
  ArgumentsBinding1_t02.c5(dynamic t1) : super.short(t1) {}

  test(dynamic t1, dynamic t2) {
    superTest(t1);
    superTestPositioned(t1);
    superTestPositioned(t2, t1);
    superTestNamed(t1);
    superTestNamed(t2, val2: t1);
    superSetter = t1;
    m = t1;
    superGetter;
  }
}

class ArgumentsBindingSuper2_t02<X> {
  X m;

  ArgumentsBindingSuper2_t02(X value): m = value {}
  ArgumentsBindingSuper2_t02.named(X value, {required X val2}): m = value {}
  ArgumentsBindingSuper2_t02.short(this.m);

  void superTest(X val) {}
  void superTestNamed(X val, {required X val2}) {}
  X get superGetter => m;
  void set superSetter(X val) {}
}

class ArgumentsBinding2_t02<X> extends ArgumentsBindingSuper2_t02<X> {
  ArgumentsBinding2_t02(X t1) : super(t1) {}
  ArgumentsBinding2_t02.c2(X t1, X t2) : super.named(t1, val2: t2) {}
  ArgumentsBinding2_t02.c5(X t1) : super.short(t1) {}

  test(X t1, X t2) {
    superTest(t1);
    superTestNamed(t2, val2: t1);
    superSetter = t1;
    m = t1;
    superGetter;
  }
}

test<T>(T t0Instance) {
  if (t0Instance is S) {
    ArgumentsBinding1_t02 c1 = new ArgumentsBinding1_t02(t0Instance);
    c1 = new ArgumentsBinding1_t02.c1(t0Instance);
    c1 = new ArgumentsBinding1_t02.c2(t1Instance, t0Instance);
    c1 = new ArgumentsBinding1_t02.c3(t0Instance);
    c1 = new ArgumentsBinding1_t02.c4(t1Instance, t0Instance);
    c1 = new ArgumentsBinding1_t02.c5(t0Instance);

    c1.test(t0Instance, t1Instance);
    c1.superTest(t0Instance);
    c1.superTestPositioned(t0Instance);
    c1.superTestPositioned(t1Instance, t0Instance);
    c1.superTestNamed(t0Instance);
    c1.superTestNamed(t1Instance, val2: t0Instance);
    c1.superSetter = t0Instance;
    c1.superGetter;

    // Test type parameters
    ArgumentsBinding2_t02<Object> c2 =
      new ArgumentsBinding2_t02<Object>(t0Instance);
    c2 = new ArgumentsBinding2_t02<Object>.c2(t1Instance, t0Instance);
    c2 = new ArgumentsBinding2_t02<Object>.c5(t0Instance);

    c2.test(t0Instance, t1Instance);
    c2.superTest(t0Instance);
    c2.superTestNamed(t1Instance, val2: t0Instance);
    c2.superSetter = t0Instance;
    c2.superGetter;
  }
}

main() {
  test<X>(t0Instance);
}
