// Copyright (c) 2019, the Dart project authors.  Please see the AUTHORS file
// for details. All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

/// @assertion We say that a type T0 is a subtype of a type T1 (written T0 <: T1)
/// when:
/// Right Object: if T1 is Object then:
///  - if T0 is an unpromoted type variable with bound B then T0 <: T1 iff
///      B <: Object
///  - if T0 is a promoted type variable X & S then T0 <: T1 iff S <: Object
///  - if T0 is FutureOr<S> for some S, then T0 <: T1 iff S <: Object.
///  - if T0 is S* for any S, then T0 <: T1 iff S <: T1
///  - if T0 is Null, dynamic, void, or S? for any S, then the subtyping does not
///      hold (per above, the result of the subtyping query is false).
///  - Otherwise T0 <: T1 is true.
/// @description Check that if T0 is a promoted type variable X & S and
/// S <: Object then T0 is subtype of Object
/// @author sgrekhov@unipro.ru
///
/// @description Check that if type T0 is a subtype of a type T1, then instance
/// of T0 can be assigned to the class member of type T1
/// @author sgrekhov@unipro.ru
///
/// This test is generated from test_types/right_object_A02.dart and
/// test_cases/class_member_x01.dart. Don't modify it!
/// If you need to change this test, then change one of the files above and then
/// run generator/generator.dart to regenerate the tests.

class X {}
class S extends X {}

S t0Instance = new S();

class ClassMember1_t01 {
  static Object s = t0Instance;
  Object m = t0Instance;
  Object _p = t0Instance;

  ClassMember1_t01() {
    s = t0Instance;
    m = t0Instance;
    _p = t0Instance;
  }

  ClassMember1_t01.named(Object value) {
    s = value;
    m = value;
    _p = value;
  }

  ClassMember1_t01.short(this.m, this._p);

  test() {
    s = t0Instance;
    m = t0Instance;
    _p = t0Instance;
  }

  set setter(Object val) {
    _p = val;
  }

  Object get getter => _p;

  static staticTest() {
    s = t0Instance;
  }

  static set staticSetter(Object val) {
    s = val;
  }

  static Object get staticGetter => t0Instance;
}

test<T>(T t0Instance) {
  if (t0Instance is S) {
    ClassMember1_t01 c1 = new ClassMember1_t01();
    c1 = new ClassMember1_t01.short(t0Instance,
        t0Instance);
    c1 = new ClassMember1_t01.named(t0Instance);
    c1.m = t0Instance;
    c1.test();
    c1.setter = t0Instance;
    c1.getter;

    ClassMember1_t01.s = t0Instance;
    ClassMember1_t01.staticTest();
    ClassMember1_t01.staticSetter = t0Instance;
    ClassMember1_t01.staticGetter;
  }
}

main() {
  test<X>(t0Instance);
}
