// Copyright (c) 2019, the Dart project authors.  Please see the AUTHORS file
// for details. All rights reserved. Use of this source code is governed by a
// BSD-style license that can be found in the LICENSE file.

/// @assertion We say that a type T0 is a subtype of a type T1 (written T0 <: T1)
/// when:
/// Right Object: if T1 is Object then:
///  - if T0 is an unpromoted type variable with bound B then T0 <: T1 iff
///      B <: Object
///  - if T0 is a promoted type variable X & S then T0 <: T1 iff S <: Object
///  - if T0 is FutureOr<S> for some S, then T0 <: T1 iff S <: Object.
///  - if T0 is S* for any S, then T0 <: T1 iff S <: T1
///  - if T0 is Null, dynamic, void, or S? for any S, then the subtyping does not
///      hold (per above, the result of the subtyping query is false).
///  - Otherwise T0 <: T1 is true.
/// @description Check that if T0 is a promoted type variable X & S and
/// S <: Object then T0 is subtype of Object
/// @author sgrekhov@unipro.ru

class X {}
class S extends X {}

S t0Instance = new S();
Object t1Instance = new Object();

const t1Default = const Object();

//# @T0 = S
//# @T1 = Object

//# <!-- Global variables & classes definition -->

test<T>(T t0Instance) {
  if (t0Instance is S) {
    //# <!-- Test body -->
  }
}

main() {
  test<X>(t0Instance);
}
